# -*- coding: utf-8 -*-

import os, sys
from PyQt4 import QtCore, QtGui
from PyQt4.Qt import Qt
import signal
import logging

from ui.select_config_ui import Ui_SelectConfigForm


try:
    _fromUtf8 = QtCore.QString.fromUtf8
except AttributeError:
    _fromUtf8 = lambda s: s

DEFAULT_DIRS = ['Descargas', 'Documentos', 'Escritorio']

class SelectConfigWindow(QtGui.QWidget):
    # Señal que se lanza cuando se finaliza la selección
    selected_list = QtCore.pyqtSignal(list)

    def __init__(self, parent=None):
        super(SelectConfigWindow, self).__init__(parent)

        self.ui = Ui_SelectConfigForm()
        self.ui.setupUi(self)

        # evitamos que la ventana pueda cerrarse
        QtCore.QObject.connect(self.ui.acceptButton,
                               QtCore.SIGNAL("clicked(bool)"),
                               self.accept)
        self.selected_list.connect(self.recv_end_signal)

        screen = QtGui.QDesktopWidget().screenGeometry()
        size = self.geometry()
        self.move((screen.width()-size.width())/2, (screen.height()-size.height())/2)

        flags = self.windowFlags() | QtCore.Qt.Window | QtCore.Qt.WindowStaysOnTopHint | QtCore.Qt.CustomizeWindowHint  | QtCore.Qt.WindowSystemMenuHint & ~ QtCore.Qt.WindowCloseButtonHint
        self.setWindowFlags(flags)

    def closeEvent(self,event):
        event.ignore()

    def set_path(self, path='./'):
        # Archivos a comprobar:
        #
        # Thunderbird:    .thunderbird
        # Chromium:       .config/chromium
        # Firefox         .mozilla

        self.path = path

        self.configs = []

        if os.path.exists( os.path.join(path, '.thunderbird') ):
            self.configs.append('thunderbird')
        else:
            self.ui.checkBox_thunderbird.hide()

        if os.path.exists( os.path.join(path, '.config/chromium') ):
            self.configs.append('chromium')
        else:
            self.ui.checkBox_chromium.hide()

        if os.path.exists( os.path.join(path, '.mozilla') ):
            self.configs.append('firefox')
        else:
            self.ui.checkBox_firefox.hide()

    def there_is_configs(self):
        return ( len(self.configs) > 0 )

    def accept(self):
        #TODO: obetner los elementos seleccionados...
        selectedItems = []

        if self.ui.checkBox_thunderbird.isChecked():
            selectedItems.append('.thunderbird')

        if self.ui.checkBox_chromium.isChecked():
            selectedItems.append('.config/chromium')

        if self.ui.checkBox_firefox.isChecked():
            selectedItems.append('.mozilla')

        self.selected_list.emit(selectedItems)

    def recv_end_signal(self, selected):
        self.hide()
        logging.debug("Se selecciona para migrar: %s" %selected)

if __name__ == '__main__':
    logging.basicConfig(level=logging.DEBUG,
                        format='%(levelname)s %(asctime)-15s %(module)s %(filename)s:%(lineno)d - %(message)s')
    from getpass import getuser

    signal.signal(signal.SIGINT, signal.SIG_DFL)

    app = QtGui.QApplication(sys.argv)

    app_gui = SelectConfigWindow()
    #app_gui.set_path('/home/%s/empty' %(getuser()))
    app_gui.set_path('/home/%s' %(getuser()))
    app_gui.show()

    sys.exit(app.exec_())
