#!/usr/bin/python
# -*- coding: utf-8 -*-

import sys
import os
import shutil
import logging
import signal
import optparse
import socket
import traceback
import getpass
from os.path import expanduser
from subprocess import Popen

from PyQt4 import QtGui, QtCore
from threading import Thread, Event
from dbus.mainloop.qt import DBusQtMainLoop
from pybytes import bytes2human

from pillbug_client import PeriodicSignal, PillBugClient
from UI_mig.select_files_window import SelectFilesWindow
from UI_mig.select_config_window import SelectConfigWindow
from UI_mig.DialogWindow import DialogWindow
from UI_mig.notification import NotifyMsg
from UI_mig.power_off import Powering_off
from UI_mig.log_out import Log_out
from ldap_user import LdapUser
from session_handler import close_session

from stat import *

try:
    _fromUtf8 = QtCore.QString.fromUtf8
except AttributeError:
    _fromUtf8 = lambda s: s

def configure_logging(debug=False):
    import logging.config
    user_log_path = os.path.join( os.path.expanduser("~"), ".cga")
    user_log_file = "ticbox_applet.log"
    if (not os.path.exists(user_log_path)):
        os.mkdir(user_log_path)
    log_file_name = os.path.join(user_log_path, user_log_file)

    if debug:
        level = 'DEBUG'
        handlers = ['default', 'tofile']
    else:
        level = 'INFO'
        handlers = ['tofile']

    log_dict_config = {
                          'version': 1,
                          'formatters': {
                              'custom': {
                                  'format': '%(levelname)s %(asctime)-15s %(filename)s %(funcName)s:%(lineno)d - %(message)s'
                              },
                          },
                          'handlers': {
                              'default': {
                                  'level': 'DEBUG',
                                  'class': 'logging.StreamHandler',
                                  'formatter': 'custom'
                              },
                              'tofile': {
                                  'level': level,
                                  'class': 'logging.handlers.RotatingFileHandler',
                                  'filename': log_file_name,
                                  'maxBytes': 1048576,
                                  'backupCount': 5,
                                  'formatter': 'custom'
                              },
                           },
                           'loggers': {
                               '': {
                                   'level': level,
                                   'handlers': handlers,
                                   'propagate': True
                               }
                           }
                        }
    logging.config.dictConfig(log_dict_config)

def touch(fname, times=None):
    fhandle = open(fname, 'a')
    try:
        os.utime(fname, times)
    finally:
        fhandle.close()

class SysTrayIcon(QtGui.QWidget):
    request_user_quota_signal = QtCore.pyqtSignal()
    do_migration_signal = QtCore.pyqtSignal(list, list)
    request_sync_signal = QtCore.pyqtSignal(bool)
    request_set_user_migration = QtCore.pyqtSignal()

    animate_icon_signal = QtCore.pyqtSignal()

    def __init__(self, parent=None):
        super(SysTrayIcon, self).__init__(parent)
        self.dbus = PillBugClient()
        # conectamos las señales a dbus
        self.request_user_quota_signal.connect(self.dbus.get_user_data)
        self.do_migration_signal.connect(self.dbus.initial_sync_data)
        self.request_sync_signal.connect(self.dbus.sync_data)
        self.request_set_user_migration.connect(self.dbus.set_user_migration)

        self.notify_msg = lambda title: NotifyMsg(title, dialog="dialog-information-symbolic")
        self.notify_error = lambda title, msg: NotifyMsg(title,
                                                         msg,
                                                         dialog="process-error-symbolic")
        self.register_handlers()
        self.home_dir = self.dbus.get_home_dir()
        self.synchro_path = os.path.join(self.home_dir, 'Documentos')
        self.file_user_is_public = os.path.join(self.synchro_path, '.user_is_public')

        self.quota_total = 0.0
        self.quota_used = 0.0

        self.user_name = getpass.getuser()
        self.profile = "fijo"
        if self.home_dir.startswith('/home/nfs/'):
            self.profile = _fromUtf8("móvil")

        # con este atributo se controla que hemos realizado la sincronización inicial,
        # la migración al nuevo sistema
        self.last_progress = 100
        self.animate_dir = True
        self.animate_index = 1

        self.createSystrayIcon()
        self.request_user_quota_signal.emit()
        if self.dbus.is_movil_user():
            if not self.is_migration_service_available():
                self.log_out = Log_out(60)
                logging.error("Imposible migracion, se pide logout")
                self.log_out.set_timeout_accept_handler(close_session)
                self.log_out.start_timeout()
            else:
                logging.info("Sincronización inicial")
                self.start_migration_proccess()
                self.updateUI()
        else:
            # Por si venimos de una sincronización inicial y necesitamos
            # restaurar la configuración:
            if self.dbus.restore_config_dirs():
                self.notify_msg(title="Se ha restaurado su configuración en este equipo por última vez.")
            self.dbus.set_synch_active(True)
            self.update_rsync(True)


        self.systrayIcon.show()

    def createSystrayIcon(self):

        self.user_info = QtGui.QAction("Esperando info...",
                                       self)
        self.user_info.setVisible(True)

        self.show_quota_action = QtGui.QAction("--",
                                               self)
        self.show_quota_action.setVisible(False)
        self.update_action     = QtGui.QAction("Actualizar ahora",
                                               self,
                                               triggered=self.update_rsync)
        self.update_action.setEnabled(False)
        self.open_dir_action   = QtGui.QAction("Abrir carpeta Documentos",
                                               self,
                                               triggered=self.open_dir)

        self.privateuser       = QtGui.QAction("Usuario privado", self,
                                               checkable=True,
                                               checked=self.is_private_user(),
                                               triggered=self.privatice_user)

        self.passwordreset     = QtGui.QAction(u"Cambiar contraseña",
                                               self,
                                               triggered=self.passwordresetaction)


        self.animation = PeriodicSignal(0.12, self.animate_icon_signal)

        self.systrayIconMenu = QtGui.QMenu(self)

        self.systrayIconMenu.addAction(self.user_info)
        self.systrayIconMenu.addAction(self.passwordreset)
        self.systrayIconMenu.addAction(self.show_quota_action)
        self.systrayIconMenu.addSeparator()
        self.systrayIconMenu.addAction(self.open_dir_action)
        self.systrayIconMenu.addSeparator()
        self.systrayIconMenu.addAction(self.privateuser)
        self.systrayIconMenu.addSeparator()
        self.systrayIconMenu.addAction(self.update_action)

        self.systrayIcon = QtGui.QSystemTrayIcon(self)
        self.systrayIcon.setContextMenu(self.systrayIconMenu)
        self.systrayIcon.setIcon(QtGui.QIcon("icons/waiting/iconw1.png"))
        self.systrayIcon.setVisible(True)

    ## Se conectan las señales
    #
    def register_handlers(self):
        self.animate_icon_signal.connect(self.animate_icon)
        # señal para que se muestre el estado del proceso de sincronizacion
        self.dbus.sync_started_signal.connect(self.show_syncro_status)
        # esta señal hará que se muestre la quota
        self.dbus.quota_data_signal.connect(self.handler_quota)
        # cuando se emite el estado de la cuota se actualiza la info
        self.dbus.status_signal.connect(self.handler_status)
        # actualizar el estado del proceso de sincronizacion
        self.dbus.progress_signal.connect(self.handler_progress)
        # manejador para que cuando se finalice la migracion se reinicie el equipo
        self.dbus.migration_ended.connect(self.migration_ended_handler)


    def handler_quota(self, used, total):

        self.quota_total = float(total) #in Kbytes
        self.quota_used = float(used)
        self.updateUI()


    ## comportamiento para usuario movil --> ha de migrarse
    # muestra ventanas y deja preparado para que el usuario elija que
    # quiere migrar. Cuando el usuario haga siguiente -> se llama a
    # self.mobile_user_migration
    def start_migration_proccess(self):
        self.notify_msg(title="Iniciando proceso de migracion")
        logging.debug("se inicia la migración de móvil a fijo")
        self.info_window = DialogWindow()
        self.mig_window_files = SelectFilesWindow()
        self.mig_config_window = SelectConfigWindow()

        # damos al usuario el path
        self.dbus.mount()
        mount_path = self.dbus.get_mounted_path()
        self.mig_window_files.set_path(mount_path)
        self.mig_config_window.set_path(mount_path)
        self.dbus.umount()

        self.info_window.accepted_signal.connect(self.mig_window_files.show)
        self.info_window.abort_signal.connect(close_session)
        self.info_window.show()

        self.migration_selected_files = []
        self.migration_selected_configs = []

        self.mig_window_files.selected_list.connect(self.user_selected_files)
        self.mig_config_window.selected_list.connect(self.user_selected_config)

        if not self.mig_config_window.there_is_configs():
            self.mig_window_files.set_final_button_msg()


    def user_selected_files(self, selected):
        self.migration_selected_files = selected

        if self.mig_config_window.there_is_configs():
            self.mig_config_window.show()
        else:
            self.mobile_user_migration()


    def user_selected_config(self, configs):
        self.migration_selected_configs = configs
        self.mobile_user_migration()


    def mobile_user_migration(self):

        logging.info("Migrando al home: %s" % self.home_dir )
        # jr comenta las siguientes 4 lineas para no migrar
        self.dbus.set_synch_active(True)
        self.do_migration_signal.emit(self.migration_selected_files,
                                              self.migration_selected_configs)
        self.update_rsync()


    ## Este handler controlará si el proceso de migracion ha terminado
    # de manera satisfactoria o no. Iniciando el proceso de reinicio
    def migration_ended_handler(self, uid, status):
        if not status:
            logging.error("el proceso de reinicio no terminó con exito")
        self.power_off = Powering_off()
        # conecto la señal del demonio con el inicio del timer
        # será el demonio quien pida reiniciar
        self.dbus.request_to_reboot.connect(self.power_off.start_timer)
        # cuando el timer termine, que llame a reboot
        self.power_off.timeout_expired_signal.connect(self.dbus.reboot)
        # SEÑAL: decimos al demonio que empiece a sincronizar!
        self.request_set_user_migration.emit()


    def handler_status(self, status):
        logging.debug("Status recibido: %d" % status)
        if status == 0: #Todo acabó bien...
            self.update_action.setText("Sincronizar ahora")
            self.update_action.setEnabled(True)
            self.animation.stop()
            self.animation = PeriodicSignal(0.12, self.animate_icon_signal)
            self.animate_dir = True
            self.animate_index = 1

            logging.info("Sincronización terminada con éxito.")
            self.update_action.setEnabled(True)
            self.request_user_quota_signal.emit()
        #Hay algún error
        else:
            self.show_error_status(status)

    def update_rsync(self, initial=False):
        self.notify_msg(title="Iniciando sincronización")
        self.request_sync_signal.emit(initial)

    def show_syncro_status(self):
        self.update_action.setDisabled(True)
        self.update_action.setText("Sincronizando...")
        self.animation.start()

    def handler_progress(self, done, total):
        self.update_action.setText("Sincronizando archivo %d de %d" % (done, total))

    def show_error_status(self, error_code):
        self.update_action.setText("Sincronizar ahora")
        self.update_action.setEnabled(True)
        self.animation.stop()
        self.animation = PeriodicSignal(0.12, self.animate_icon_signal)
        self.systrayIcon.setIcon(QtGui.QIcon("icons/icon_error.png"))

        if not self.show_quota_action.isVisible():
                self.show_quota_action.setVisible(True)

        if error_code == 2:
            self.show_quota_action.setText("Error: necesita liberar espacio")
            self.notify_error(title="No se han sincronizado todos los archivos",
#                              msg="Necesita liberar espacio")
			      ### A petición de coordinación se modifica el mensaje de advertencia ###
                              msg="Ha superado la cuota de usuario, los últimos cambios no han sido guardados. Use otro medio de almacenamiento para guardarlos.")
        else:
            self.show_quota_action.setText(_fromUtf8("Error en la conexión"))
            self.notify_error(title="Error en la sincronización",
                                msg="Error conectando con el servidor")


    def animate_icon(self):
        if self.animate_index < 10:
            self.animate_index = self.animate_index + 1
        else:
            self.animate_index = 1

        self.systrayIcon.setIcon(QtGui.QIcon("icons/waiting/iconw%d.png" % self.animate_index))


    def updateUI(self):
        # información del usuario
        ldap = LdapUser(self.user_name)
        if ldap.is_movil():
            self.profile = _fromUtf8("móvil")
        else:
            self.profile = _fromUtf8("fijo")

        self.user_info.setText("%s (perfil %s)" %(self.user_name,
                                                 self.profile))
        # vamos con la cuota, calculamos porcentaje para mostrar en el
        # menu desplegable del applet

        if self.quota_total > 0:
            porc_usado = (self.quota_used * 100) / self.quota_total
            total = bytes2human(self.quota_total*1024)
            usado = bytes2human(self.quota_used*1024)
            logging.debug("la informacion de quota es %s/%s, usado %s porciento"
                          %(self.quota_used, self.quota_total, porc_usado))

            self.show_quota_action.setText("Usado %.0f%% (%s de %s)"
                                           % (porc_usado, usado, total))
            if not self.show_quota_action.isVisible():
                self.show_quota_action.setVisible(True)

            #El Icono...
            # (last_progress < 100) => Sincronización incompleta!
            if porc_usado > 95 or self.last_progress < 100:
                icon_index = 11
            else:
                icon_index = int(round(porc_usado/10, 0))
                if icon_index == 0:
                        icon_index = 1
            logging.debug("Se establece indice de icono: %s" %icon_index)
            self.systrayIcon.setIcon(QtGui.QIcon("icons/icon_%d.png"
                                                 % icon_index))

        else:
            self.handler_status(1)

        # Comprobamos si el usuario es privado, y establecemos el check correctamente
        self.is_private_user(True)

    def open_dir(self):
        os.system("xdg-open %s " % self.synchro_path)

    def is_private_user(self, update=False):
        """ Función que controla los permisos de la carpeta personal
        para establecer el check de la opción "Usuario privado" del  applet.
        En caso de inconsistencias, se establecen los permisos según si existe
        el archivo ~/Documentos/.user_is_public """

        perms = oct(os.stat(self.home_dir)[ST_MODE])
        logging.debug("Permisos carpeta: %s" %perms)
        public = os.path.isfile(self.file_user_is_public)
        if perms == "040700" and not public:
            if update:
                self.privateuser.setChecked(True)
            return True
        elif perms == "040700" and public:
            logging.debug("Parece ser que el usuario es público, pero la carpeta tiene permisos incorrectos, cambiamos permisos")
            os.chmod(self.home_dir, 0755)
            if update:
                self.privateuser.setChecked(False)
            return False
        elif perms == "040755" and public:
            if update:
                self.privateuser.setChecked(False)
            return False
        else:
            logging.debug("Parece ser que se ha marcado el usuario como privado, cambiamos permisos")
            os.chmod(self.home_dir, 0700)
            if update:
                self.privateuser.setChecked(True)
            return True

    def privatice_user(self):
        """ Función que "privatiza" un usuario, según si antes era
        privado o no. Esta función controla el fichero ~/Documentos/.user_is_public """

        if self.is_private_user()==True:
            logging.debug("Desprivatizando usuario")
            os.chmod(self.home_dir, 0755)
            touch(self.file_user_is_public)
            self.request_sync_signal.emit(False)
            self.privateuser.setChecked(False)

        else:
            logging.debug("Privatizando usuario")
            os.chmod(self.home_dir, 0700)
            try:
                os.remove(self.file_user_is_public)
                self.request_sync_signal.emit(False)
            except:
                logging.error("Fichero %s no encontrado (¿primera sincronización?)" % self.file_user_is_public)

            finally:
                self.privateuser.setChecked(True)

    def passwordresetaction(self):
        """ Llamada al script de reset de contraseñas """

        Popen(['/usr/lib/python2.7/dist-packages/cgapasswordreset/cgapasswordresetgui.py'])

    def is_migration_service_available(self, host="f0", port=30000):
        s =  socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        s.settimeout(0.5)
        result = s.connect_ex((host, port))
        s.close()
        logging.debug("Socket.Connect returns code %s" %result)
        if 0 == result:
            result = True
        else:
            logging.debug("El servicio de migración en %s:%s no está activo"
                          %(host, port))
            result = False
        return result


if __name__ == '__main__':
    parser = optparse.OptionParser()
    parser.add_option('--debug', help='start in debug mode',
                      dest="debug", action="store_true", default=False)
    (options, args) = parser.parse_args()

    configure_logging(options.debug)

    try:
        DBusQtMainLoop(set_as_default=True)
        app = QtGui.QApplication([])
        signal.signal(signal.SIGINT, signal.SIG_DFL)
        systrayicon = SysTrayIcon()

        app.exec_()
    except Exception:
        logging.critical("Excepción:\n %s" % traceback.format_exc())
        sys.exit(1)
