# -*- coding: UTF-8 -*-

import ldap
import getpass
import socket
import sys

import logging
from repr import repr


class LdapUser:
    
    def __init__(self, uid, ldap_server="ldap"):
        self.ldap_server = ldap_server
        self.uid = uid
        base = self.get_base()
        self.baseDN = "uid=%s,ou=People,%s" % (self.uid, base)
        
        self.homeDirectory = self.request_home_dir()

    def is_movil(self):
        return self.homeDirectory.startswith('/home/nfs/')
    
    def get_home_dir(self):
        self.homeDirectory = self.request_home_dir()
        return self.homeDirectory
    
    def request_home_dir(self):
        lcon = ldap.open(self.ldap_server)
        search_id = lcon.search(self.baseDN, ldap.SCOPE_SUBTREE)
        result_type, result_data = lcon.result(search_id, 0)
        
        
        for f_key, f_val in result_data:
            for key, val in f_val.items():
                if key == 'homeDirectory':
                    return val[0]

    def get_base(self):
        ldap_conf = open('/etc/ldap.conf').readlines()
        if len(ldap_conf) > 0:
            ldap_conf.reverse()
            for line in ldap_conf:
                if line.startswith('base '):
                    result = line.replace('base ', "").strip()
                    logging.debug("se devuelve como base:   -%s-" %result)
                    return result
        return ''
    
    
    """
    Esta función realiza se conecta con @ldap_host para realizar la migración del perfil.
     parametros para conectar con ldap en f0
      el socket escucha por puerto 30000
    """
    def set_user_migration(self, ldap_host='f0', port=30000):
        # comprobamos si hay conectividad con el equipo
        if( self.test_connection(ldap_host, port)==True):
            logging.debug("hay conexión con %s:%s" %(ldap_host, port))
            result = self.__send_command(self.uid, ldap_host, port)
            return True
        else:
            logging.info("imposible conectar con %s por el puerto %s"
                             %(ldap_host, port))
            return False
    
    
    def test_connection(self, target_host, port):
        s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        s.settimeout(0.5)
        result = -1
        #connect_ex no da excepciones, devuelve el valor 
        result = s.connect_ex((target_host, port))
        logging.debug("Socket.Connect returns code %s" %result)
        s.close()
        return result==0
    
    
    def __send_command(self, command, host, port):
        res    = 1
        tries = 3
        while ( tries > 0 ):
            try:
                s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
                s.connect((host, port))
                logging.debug("Sent from %s" %( repr(command)))
                returned = s.send(command)
                #logging.debug("The call send returns %s (bytes sent?)" %repr(returned))
                res = 0
                break
            except Exception as e:
                logging.error( "Error sending command %s to %s:%s \nReason: %s"
                                    %(command, host, port, str(e)))
            finally:
                s.close()
                tries -= 1
        if( tries==0 and res!=0):
            logging.error("After 3 tries the request can't be sent %s:%s" %(host,port))
        return res
    
if __name__ == "__main__1":
    logging.basicConfig(level=logging.DEBUG)
    import getpass
    import sys
    uid= sys.argv[1]
    logging.debug("buscando a usuario con id=%s" %uid)
    luser = LdapUser(uid)
    if luser.is_movil():
        s = "skyrim"
        while s.lower()!="s" and s.lower()!="n":
            s = raw_input('usuario %s es movil, quierer migrarlo? S/n' %uid)
            if s.lower() == "s":
                result = luser.set_user_migration()
                if not result:
                    logging.error('no se ha conseguido migrar al usuario %s'
                            %uid)
    else:
        logging.info("usuario %s es fijo" %uid)

    
    
    