#!/usr/bin/env python
# -*- coding: UTF-8 -*-

import os, getpass
import dbus
import dbus.service
import pyinotify
import logging
from PyQt4 import QtCore
import subprocess
from threading import Thread, Event
import time

from ldap_user import LdapUser
from dbus.exceptions import DBusException


class PillBugClient(QtCore.QObject):
    sync_started_signal = QtCore.pyqtSignal()
    quota_data_signal = QtCore.pyqtSignal(str, str)
    status_signal = QtCore.pyqtSignal(int)
    progress_signal = QtCore.pyqtSignal(int, int)
    request_to_reboot = QtCore.pyqtSignal(bool)
    migration_ended = QtCore.pyqtSignal(int, int)

    def __init__(self):
        QtCore.QObject.__init__(self, None)
        self.__bus = dbus.SystemBus()
        self.pillbug_service = None
        while self.pillbug_service == None:
            try:
                prx = self.__bus.get_object('org.cga.ticbox.pillbug',
                                            '/org/cga/ticbox/pillbug')
                self.pillbug_service = dbus.Interface(prx,
                                                       dbus_interface='org.cga.ticbox.pillbug')
            except dbus.DBusException, e:
                logging.info("Duermo 3 segundos esperando al servicio pillbug")
                time.sleep(3)
            else:
                logging.info("Conectado con servicio pillbug")
                break

        self.dbus_signals = DBusSignals('/org/cga/ticbox/pillbug')
        self.__bus.add_signal_receiver(self.status_handler,
                                'sync_status',
                                'org.cga.ticbox.pillbug')
        self.__bus.add_signal_receiver(self.progress_handler,
                                'sync_progress',
                                'org.cga.ticbox.pillbug')

        self.__bus.add_signal_receiver(self.migration_ended_handler,
                                       'migration_finished',
                                       'org.cga.ticbox.pillbug')

        self.__in_progress = False
        self.__synch_active = False

        self.user = {}
        self.user['name'] = getpass.getuser()
        self.user['uid'] = os.getuid()
        self.user['gid'] = os.getgid()

        self.luser = LdapUser(self.user['name'])
        self.user['home'] = self.luser.get_home_dir()

        #Pyinotify file monitor...
        wm1 = pyinotify.WatchManager()
        # Stats is a subclass of ProcessEvent
        s1 = pyinotify.Stats()

        self.file_monitor = FileMonitor(s1)
        self.file_monitor.sync_monitor_signal.connect(self.sync_data)

        notifier1 = pyinotify.ThreadedNotifier(wm1,
                                               default_proc_fun=self.file_monitor)
        notifier1.start()

        monitor_dir = "%s/Documentos" % self.get_home_dir()
        logging.debug("Monitoring: %s" % monitor_dir)
        if not os.path.exists(monitor_dir):
            os.mkdir(monitor_dir)

        flags_monitor = pyinotify.IN_CLOSE_WRITE | pyinotify.IN_DELETE | pyinotify.IN_MOVED_FROM | pyinotify.IN_MOVED_TO | pyinotify.IN_CREATE
        wm1.add_watch(monitor_dir, flags_monitor, rec=True, auto_add=True)

        #Además, lanzar cada 2 min la actualización...
        self.sync_period = PeriodicSignal(120,
                                          self.file_monitor.sync_monitor_signal,
                                          args=[False])
        self.sync_period.start()

    def is_movil_user(self):
        self.luser = LdapUser(self.user['name'])
        return self.luser.is_movil()

    def set_user_migration(self):
        result = self.luser.set_user_migration()
        result = True
        if result:
            logging.debug("Migración exitosca")
            self.request_to_reboot.emit(True)
        else:
            #TODO: cambiar la gestión de la señal par que la lógica vaya a applet_quota
            logging.debug("problemas realizando la migración de perfil con ldap")

    def get_home_dir(self):
        return self.user['home']

    ## Activa o desactiva la sincronizacion
    #
    def set_synch_active(self, is_active):
        self.__synch_active = is_active
    
    
    def get_user_data(self):
        logging.debug("   PillBugClient => get_user_data")
        try:
            quota = self.pillbug_service.get_user_data(self.user['name'],
                                                        self.user['uid'],
                                                        self.user['gid'],
                                                        self.user['home'])
        except DBusException as e:
            logging.error("excepcion en llamada a dbus!")
            self.status_signal.emit(1) #Error, no hay cuota!
        else:
            if quota[1] > 0:
                self.quota_data_signal.emit("%d" % quota[0], "%d" % quota[1])
            else:
                self.status_signal.emit(1) #Error, no hay cuota!
    
    
    def initial_sync_data(self, elements_to_preserve, configs_to_preserve):
        if not self.__in_progress:
            self.__in_progress = True
            logging.debug("   PillBugClient => initial_sync_data")
            
            self.dbus_signals.initial_sync_data(self.user['name'],
                                                self.user['uid'],
                                                self.user['gid'],
                                                self.user['home'],
                                                elements_to_preserve,
                                                configs_to_preserve)
            
            self.sync_started_signal.emit()
    
    
    ## Llama a la señal del objeto dbus daemon solo si no hay otra sincronziacion
    # activa.Para que se envie tambien es necesario que la sincronizacion este
    # activa (metodo set_synch_active)
    def sync_data(self, initial=False):
        if not self.__in_progress and self.__synch_active:
            self.__in_progress = True
            logging.info("Iniciando sincronización.")
            
            self.dbus_signals.sync_data(self.user['name'],
                                        self.user['uid'],
                                        self.user['gid'],
                                        self.user['home'],
                                        initial)
            
            self.sync_started_signal.emit()
    
    
    def status_handler(self, uid, status):
        logging.debug("   PillBugClient => status_handler(%d)" % status)
        if uid == self.user['uid']:
            self.__in_progress = False
            self.status_signal.emit(status)

    def progress_handler(self, uid, done, total):
        logging.debug("   PillBugClient => progress_handler(%d / %d)" % (done, total) )
        if uid == self.user['uid']:
            self.progress_signal.emit(done, total)

    def migration_ended_handler(self, uid, status):
        logging.debug("migracion terminada de uid:%s con codigo:%s"
                      %(uid, status))
        self.migration_ended.emit(uid, status)

    def reboot(self):
        self.pillbug_service.reboot()

    def get_mounted_path(self):
        return self.pillbug_service.get_inicial_path(  self.user['name'],
                                                        self.user['uid'],
                                                        self.user['gid'],
                                                        self.user['home'])

    def mount(self):
        #TODO: modificar metodo dbus para añadir valor devuelto
        logging.debug("se solicita mount desde cliente")
        try:
            self.pillbug_service.mount(self.user['name'],
                                        self.user['uid'],
                                        self.user['gid'],
                                        self.user['home'])
        except Exception as e:
            logging.error("ERROR solicitando montaje")
            logging.error( traceback.format_exc())

    def umount(self):
        #TODO: modificar metodo dbus para añadir valor devuelto
        logging.debug("se solicita mount desde cliente")
        try:
            self.pillbug_service.umount(self.user['name'],
                                        self.user['uid'],
                                        self.user['gid'],
                                        self.user['home'])
        except Exception as e:
            logging.eror("ERROR solicitando desmontaje")
            logging.error( traceback.format_exc())


    def restore_config_dirs(self):
        res_copy = self.pillbug_service.restore_config_dirs(self.user['name'],
                                                        self.user['uid'],
                                                        self.user['gid'],
                                                        self.user['home'])
        if res_copy:
            # caso de cierto, algo mayor que cero
            logging.info("Se ha restaurado configuración...")
        else:
            logging.debug("Como siempre (menos una vez), no se ha tenido que restaurar configuración alguna")

        file_name = os.path.join( os.path.expanduser("~"),
                                  "backup-restore-gconf.xml")
        logging.debug("restaurando configuracion gconf desde %s" %file_name)

        return res_copy


class DBusSignals(dbus.service.Object):
    def __init__(self, object_path):
        dbus.service.Object.__init__(self, dbus.SystemBus(), object_path)

    ##
    #
    @dbus.service.signal(dbus_interface='org.cga.ticbox.pillbug',
                         signature='suusasas')
    def initial_sync_data(self, user_name, uid, gid, home_dir, element_list, config_list):
        logging.debug("Emitiendo dbus -> initial_sync_data: sync_data")

    @dbus.service.signal(dbus_interface='org.cga.ticbox.pillbug',
                         signature='suusb')
    def sync_data(self, user_name, uid, gid, home_dir, is_initial):
        logging.debug("Emitiendo dbus -> sync_data: sync_data")


class FileMonitor(pyinotify.ProcessEvent, QtCore.QObject):
    sync_monitor_signal = QtCore.pyqtSignal(bool)

    def __init__(self, stats):
        QtCore.QObject.__init__(self, None)
        super(type).__init__(type(stats))
        self.retarded = RetardedProccess(4, self.sync_monitor_signal,[False])

    def process_default(self, event):
        # Si recibimos una ristra de cambios en el directorio,
        # debemos esperarnos a la última señal del sync_monitor para lanzar 
        # self.sync_data...
        if (not event.name.startswith('.') and not event.name.endswith('~') 
            and event.name != ''):
            logging.debug('Monitor (is_dir: %s): %s -> %s' % (event.dir,
                                                              event.maskname,
                                                              event.name))
            self.retarded.stop()
            self.retarded = RetardedProccess(3, self.sync_monitor_signal, [False])
            self.retarded.start()


class PeriodicSignal(Thread):

    def __init__(self, interval, signal, args=[]):
        Thread.__init__(self)
        self.interval = interval
        self.signal = signal
        self.args = args
        self.finished = Event()
        logging.debug("se crea el temporizador!")

    def run(self):
        logging.debug("Thread running")
        while not self.finished.is_set():
            self.finished.wait(self.interval)
            if not self.finished.is_set():
                self.signal.emit(*self.args)

    def stop(self):
        logging.debug("Thread stopped")
        self.finished.set()

class RetardedProccess(Thread):

    def __init__(self, interval, signal, args):
        Thread.__init__(self)
        self.interval = interval
        self.signal = signal
        self.args = args
        self.finished = Event()

    def run(self):
        self.finished.wait(self.interval)
        if not self.finished.is_set():
            self.signal.emit(*self.args)

    def stop(self):
        self.finished.set()
