#!/usr/bin/python
# -*- coding:utf-8 -*-

import dbus
import dbus.service
import logging
import time
import traceback
import os
import gobject
import warnings

from dbus.mainloop.glib import DBusGMainLoop

from pillbug_exceptions import *
from synchronizer import Synchronizer
import subprocess


class PillBug_daemon(dbus.service.Object):

    def __init__(self):
        logging.info("Inicializando interfaz en dbus: SystemBus")
        self.bus = dbus.SystemBus()
        bus_name = dbus.service.BusName('org.cga.ticbox.pillbug', self.bus)
        dbus.service.Object.__init__( self,
                                      bus_name,
                                      '/org/cga/ticbox/pillbug')
        
        self.bus.add_signal_receiver(self.initial_sync_data,
                                     'initial_sync_data', 
                                     'org.cga.ticbox.pillbug')
        self.bus.add_signal_receiver(self.sync_data,
                                     'sync_data', 
                                     'org.cga.ticbox.pillbug')
        
        self.__migration_in_progress = False
        self.flag_file = '/tmp/.pillbug'
        
    ##
    #
    @dbus.service.method('org.cga.ticbox.pillbug')
    def reboot(self):
        os.system("/sbin/reboot")
        
        
    @dbus.service.method('org.cga.ticbox.pillbug',
                         signature='suus',out_signature='s'
                         )
    def get_inicial_path(self, name, uid, guid, home_dir):
        s = Synchronizer(name, uid, guid, home_dir)
        return s.mount_point
    
    #FIXME: modificar esto, no es necesario montar. Implementar para habilitar 
    # dbus para enviar la informacion de lo que hay en la carpeta de nfs
    @dbus.service.method('org.cga.ticbox.pillbug',
                         signature='suus')
    def mount(self, name, uid, guid, home_dir):
        s = Synchronizer(name, uid, guid, home_dir)
        s.mount()
        
    @dbus.service.method('org.cga.ticbox.pillbug',
                         signature='suus')
    def umount(self, name, uid, guid, home_dir):
        s = Synchronizer(name, uid, guid, home_dir)
        s.umount()
    
    
    ## Este método devuelve la información de quota perteneciente al usuario
    # con nombre, user id y group id dados.
    @dbus.service.method('org.cga.ticbox.pillbug',
                         signature='suus',out_signature='au')
    def get_user_data(self, name, uid, guid, home_dir):
        logging.debug("Se ejecuta get_user_data con argumentos: %s, %s, %s"
                      %(name, uid, guid))
        
        s = Synchronizer(name, uid, guid, home_dir)
        
        result = s.get_quota_info()
        logging.debug("Synchronizer devuelve los datos para %s(%s:%s) %s"
                      %(name, uid, guid, result))
        return result
    
    ##
    ##            SEÑALES
    ##    
    @dbus.service.signal('org.cga.ticbox.pillbug',
                         signature='uu')
    def migration_finished(self, uid, status):
        logging.debug("Emitiendo señal de estado con %s" %status)
    
    
    @dbus.service.signal('org.cga.ticbox.pillbug',
                         signature='uu')
    def sync_status(self, uid, status):
        logging.debug("Emitiendo señal de estado con %s" %status)

    
    ##########################
    
    
    def initial_sync_data(self, name, uid, guid, home_dir, list_elements, 
                          list_configs):
        try:
            self.__initial_sync_handler( name, uid, guid, home_dir, 
                                         list_elements, list_configs)
        except Exception as e:
            logging.error("excepcion PUTA: %s" %e.message)
            logging.error( traceback.format_exc())
            self.sync_status(uid, 1)
        else:
            self.sync_status(uid, 0)
        
        
    ##FIXME: pensar otra manera de comprobar el estado en que termina
    # la sincronización
    def __initial_sync_handler(self, name, uid, guid, home_dir, list_elements, 
                               list_configs):
        self.__migration_in_progress = True
        s = Synchronizer(name, uid, guid, home_dir)
        s.mount()
        try:
            if len(list_configs) > 0:
                logging.debug("Guardando configuraciones...")
                s.save_config_dirs(list_configs)
            
            if len(list_elements)==0:
                logging.debug("no hay elementos que conservar")
            else:
            
                encoded_list = self.encode_element_list(list_elements, "utf-8")
                logging.debug("se conserva %s" %(encoded_list))
    
                for element in s.get_all_elements():
                    absolute_path = os.path.join(s.mount_point, element)
                    if not element in encoded_list:
                        logging.debug("a borrar %s" %absolute_path)
                        s.delete_foo(absolute_path)
                    else:
                        logging.debug("se mantiene elemento: %s" %(absolute_path))
        finally:
            s.umount()
        
        self.__sync(name, uid, guid, home_dir, initial=True)
    
    
    def sync_data(self, name, uid, guid, home_dir, initial):
        logging.info("Se solicita sincronizacion (initial=%s)de %s (%s/%s)"
                      %(initial, name, uid, guid))
        self.__sync(name, uid, guid, home_dir, initial)
    
    
    def __sync(self, name, uid, guid, home_dir, initial=False):
        self.set_flag()
        try:
            s = Synchronizer(name, uid, guid, home_dir)
            
            if not initial:
                s.rsync_from_local(progress_handler=self.sync_progress)
            else:
                s.rsync_from_nfs(progress_handler=self.sync_progress)
            # si había una sincronizacion inicial (migracion) en curso
            # emito la señal por dbus migration_finished
            if self.__migration_in_progress:
                self.__migration_in_progress=False
                self.migration_finished(uid, 0)
            self.sync_status(uid, 0)
        
        except QuotaExceeded:
            logging.debug("cuota excedida por %s" %name)
#            logging.error( traceback.format_exc())
            self.sync_status(uid, 2)
            if self.__migration_in_progress:
                self.migration_finished(uid, 1)
        except Exception as e:
            logging.error("Exception!! : %s" %e)
#            logging.error( traceback.format_exc())
            self.sync_status(uid, 1)
            if self.__migration_in_progress:
                self.migration_finished(uid, 1)
        finally:
            if self.__migration_in_progress:
                self.__migration_in_progress=False
            self.unset_flag()
            pass
        
        
    def set_flag(self):
        flag = open(self.flag_file, 'w')
        flag.write("%s" % time.time())
        flag.close()
        pass
    
    
    def unset_flag(self):
        flag = open(self.flag_file, 'w')
        flag.close()
        pass
    
    
    @dbus.service.signal('org.cga.ticbox.pillbug',
                         signature='uuu')
    def sync_progress(self, uid, done, total):
        logging.debug("Emitida señal de sincronizando a %s, en proceso %s de %s archivos" 
                      %(uid, done, total))
        
    
    def encode_element_list(self, list_elements, my_encode):
        result = []
        for element in list_elements:
            new_element = element.encode(my_encode)
#            logging.debug("se codifica element a %s, que es %s" 
#                          %(new_element, type(new_element)))
            result.append(new_element)
        return result
    
    # Se necesita hacer esto al antes de que al usuario le de tiempo de arrancar 
    # cualquier aplicación... Ñññññapaaaa!
    @dbus.service.method('org.cga.ticbox.pillbug',
                         signature='suus',out_signature='b')
    def restore_config_dirs(self, name, uid, guid, home_dir):
        s = Synchronizer(name, uid, guid, home_dir)
        return s.restore_config_dirs()
    
    