#!/usr/bin/python
# -*- coding:utf-8 -*-

import dbus.mainloop.glib
import gobject
import dbusManager.server
import optparse
import os
import signal
import sys

import logging

def configure_logging(debug=False):
    import logging.config
    log_path = "/var/log/cga"
    log_file = "ticbox.log"
    if (not os.path.exists(log_path)):
        os.mkdir(log_path)
    log_file_name = os.path.join(log_path, log_file)

    if debug:
        level = 'DEBUG'
        handlers = ['default', 'tofile']
    else:
        level = 'INFO'
        handlers = ['tofile']
        
    log_dict_config = {
                          'version': 1,
                          'formatters': {
                              'custom': {
                                  'format': '%(levelname)s %(asctime)-15s %(filename)s:%(lineno)d %(funcName)s() - %(message)s'
                              },
                          },
                          'handlers': {
                              'default': {
                                  'level': 'DEBUG',
                                  'class': 'logging.StreamHandler',
                                  'formatter': 'custom'
                              },
                              'tofile': {
                                  'level': level,
                                  'class': 'logging.handlers.RotatingFileHandler',
                                  'filename': log_file_name,
                                  'maxBytes': 1048576,
                                  'backupCount': 5,
                                  'formatter': 'custom'
                              },
                           },
                           'loggers': {
                               '': {
                                   'level': level,
                                   'handlers': handlers,
                                   'propagate': True
                               }
                           }
                        }
    logging.config.dictConfig(log_dict_config)


def main():

    dbus.mainloop.glib.DBusGMainLoop(set_as_default=True)
    daemon = dbusManager.server.PillBug_daemon()

    signal.signal(signal.SIGINT, signal.SIG_DFL)

    loop = gobject.MainLoop()
    
    try:
        loop.run()
    except Exception as e:
        logging.fatal("excepcion sin capturar en loop principal: " % e)


if __name__ == "__main__":
    
    parser = optparse.OptionParser()
    parser.add_option('--debug', help='Start in debug mode, NO DAEMONIZE',
                      dest="debug", action="store_true", default=False)
    (options, args) = parser.parse_args()
    
    configure_logging(options.debug)
    
    if options.debug:
        logging.debug("Iniciado angel-caido (demonio sin daemonizar) con pid %s" %os.getpid())
        try:
            main()
        except Exception as e:
            logging.critical("excepcion: %s" %repr(e))
    else:
        pid = os.fork()
        if pid == 0:
            #hijo
            logging.info("Iniciado demonio con pid %s" %os.getpid())
            try:
                main()
            except Exception as e:
                logging.critical("Excepcion inesperada en main: %s" %repr(e))
        else:
            #padre
            sys.exit(0)
