#!/usr/bin/env python
# -*- coding: UTF-8 -*-

import gobject
import os, os.path
import sys
import re
import subprocess
import logging
import shutil
import tempfile
import time
import tarfile

from pillbug_exceptions import *


class Synchronizer:

    def __init__(self, user_name, user_id, user_guid, home_dir):
        #TODO: ¿ es necesario comprobar que uids y demás son validos?
        self.__user_name = str(user_name)
        self.__user_id = user_id
        self.__user_guid = user_guid
        self.__home_dir = home_dir

        self.nfs_server = "c0"
        self.server_path = "/home/nfs/"+self.__user_name
        # es importante la barra del punto de montaje para que a la hora de
        # sincronizar sincronice el contenido de la carpeta fuente en ~/Documentos
        self.mount_point = os.path.join( "/var/cache/nfs/",
                                         self.__user_name+"/")
        self.ticbox_path = os.path.join( home_dir,
                                         'Documentos/')
        self.config_tmp_dir = "/usr/share/pillbug/configs"


    def mount(self):

        if not os.path.exists(self.mount_point):
            logging.info("No hay punto de montaje, se crea en %s"
                         %self.mount_point)
            os.mkdir(self.mount_point)
        try:
            if self.__is_mounted(self.mount_point):
                logging.debug("%s estaba montado y se pidio montar, se desmonta antes" %self.mount_point)
                return True
        except Exception as e:
            logging.debug("error comprobando montaje en %s" %self.mount_point)
            logging.error( traceback.format_exc())
            return False

        # si llegados aquí todo va bien, listo para montar
        command = ["mount","-t","nfs",
                   self.nfs_server+":"+self.server_path,
                   self.mount_point]

        (err_code, output) = self.execute_command(command)
        if(err_code >0):
            logging.error( "Error ejecutando mount: %s , código de error %s" %(command, err_code))
            return False
        else:
#            logging.debug( "Sistema de ficheros nfs montado en %s"
#                           %(self.mount_point))
            pass
            return True


    def umount(self):
        command = ["umount",self.mount_point]

        (err_code, output) = self.execute_command(command)

        if(err_code ==130):
            logging.error( "Nada montado en %s" %(err_code))
        elif(err_code==0):
            pass
        elif (err_code>0):
            logging.error( "Error desmontando: %s" %(err_code))
        else:
            logging.error( "Error desconocido: %s" %(err_code))


    def __is_mounted(self, mount):
        output = subprocess.check_output(["mount", "-l"])
        for line in output.split("\n"):
            #find devuelve el primer indice o -1 si no existe
            if line.find(mount) > -1:
                return True
        return False

    def execute_command_as_user( self, command_array):
        os.setegid(self.__user_guid)
        os.seteuid(self.__user_id)
        logging.debug("Me hago usuario %s con su uid(%s) y gid(%s)"
                        %(self.__user_name,
                          self.__user_id,
                          self.__user_guid))
        logging.debug("ejecuto %s" %str(command_array))
        result = self.execute_command(command_array)
        os.seteuid(0)
        os.setegid(0)
        logging.debug("Recupero mi uid(%s) y gid(%s)"
                      %(os.getuid(),
                        os.getgid()))
        return result


    def execute_command(self, command_array):
        # este será el código devuelto por DEFAAAUUULT (con voz de chiquito)
        return_code = 0
        try:
            output = subprocess.check_output(command_array)
        except subprocess.CalledProcessError as thrown_exception:
            logging.error("Error ejecutando comando (error_code: %s)" % thrown_exception.returncode)
            return_code = thrown_exception.returncode
            output = thrown_exception.output
        return (return_code, output)


    def check_ticbox(self):
        def f():
            logging.debug("Comprobando directorio %s con uid(%s) gid(%s)"
                          %(self.ticbox_path,
                            self.__user_id,
                            self.__user_guid))
            if not os.path.isdir(self.ticbox_path):
                os.mkdir(self.ticbox_path)
        self.run_as_user(f)


    def get_user_ticbox(self, username):
        def f():
            return os.path.join(os.path.expanduser("~"),"Documentos")
        return self.run_as_user(f)


    def delete_foo(self, absoulte_path):
        self.mount()
        os.setegid(self.__user_guid)
        os.seteuid(self.__user_id)
        if os.path.isdir(absoulte_path):
            shutil.rmtree(absoulte_path)
        else:
            os.remove(absoulte_path)
        os.seteuid(0)
        os.setegid(0)
        self.umount()


    def get_all_elements(self):
        self.mount()
        os.setegid(self.__user_guid)
        os.seteuid(self.__user_id)
        result = os.listdir(self.mount_point)
        os.seteuid(0)
        os.setegid(0)
        self.umount()
        logging.debug("se elegiran para borra entre %s" %result)
        return result


    def run_as_user(self, function):
        os.setegid(self.__user_guid)
        os.seteuid(self.__user_id)
        result = function()
        os.seteuid(0)
        os.setegid(0)
        return result

    #
    #    SYNCHRO    STUFFS
    #
    def rsync_from_nfs(self,progress_handler=None):
        logging.debug("sincronizando DESDE nfs")
        self.__rsync(source=self.mount_point,
                     dest=self.ticbox_path,
                     progress_handler=progress_handler,
                     delete=True)

    ## Este metodo realizará una sincronizacion con
    #     origen:  carpeta TicBox local
    #     destino: punto de montaje nfs
    def rsync_from_local(self, progress_handler=None):
        logging.debug("sincronizando HACIA nfs")
        self.__rsync( source=self.ticbox_path,
                      dest=self.mount_point,
                      progress_handler=progress_handler,
                      delete=True)


    ## Este metodo realizará una sincronizacion con
    #     origen:  punto de montaje nfs
    #     destino: carpeta TicBox local
    def __rsync(self, source, dest,  progress_handler=None, delete=False):
        rsync_command = ["rsync", "-avur", source, dest]

        if delete:
            rsync_command.append("--delete")

        self.check_ticbox()

        self.mount()
        logging.debug("Iniciando sincronizacion entre %s y %s"
                      %(source, dest))
        if not callable(progress_handler):
            (return_code, output) = self.execute_command_as_user(rsync_command)
            logging.debug("El rsync dice \"%s\", codigo devuelto %s"
                          %(output, return_code))
        else:
            os.setegid(self.__user_guid)
            os.seteuid(self.__user_id)

            to_synch_array = self.__get_total_files_to_sync(rsync_command)
            total_files = len(to_synch_array)

            process = subprocess.Popen( rsync_command,
                                       stdout=subprocess.PIPE,
                                       stderr=subprocess.PIPE)
            # marca cuando hayamos encontrado la primera línea
            flag = False
            files_synchronized = []
            while True:
                line = process.stdout.readline().strip()
                if not flag and line == 'sending incremental file list':
                    flag = True
                elif flag and not line.endswith('/') and not line.startswith('/') and total_files != 0 and len(files_synchronized) < total_files:
                    files_synchronized.append(object)
                    logging.debug("sincronizado %s" %(line))
                    # llamamos a la señal con: uid, sincronizado, total a sincronizar
                    progress_handler(self.__user_id,
                                     len(files_synchronized),
                                     total_files)
                return_code = process.poll()
                if return_code != None:
                    break

            os.seteuid(0)
            os.setegid(0)
            logging.debug(" HARD DEBUG: return_code = %s " % return_code)

        self.umount()
        if return_code == 11:
            raise QuotaExceeded
        elif return_code > 0:
            logging.info("*** rsync ha terminado con return_code = %s    ¿ERROR?" % return_code)


    def __get_total_files_to_sync(self, in_command_array):
        #cambiamos las opciones para que se ejecute con -n, dry run
        command_array = list(in_command_array)
        command_array[1] = '-navur'
        command_array.insert(0, "strace") # CAMBIO 7390
        process = subprocess.Popen( command_array,
                                     stdout=subprocess.PIPE)

        # esperamos al hijo para así contar todo
        process.wait()

        output = process.stdout.readlines()
        files_to_sync = []
        guia = False
        for element in output:
            element = element.strip()
            if element == 'sending incremental file list':
                guia = True
                pass
            elif len(element) == 0:
                break
            elif  guia and not element.endswith('/') and not element.startswith('/'):
                logging.debug("*** ELEM: %s " % element)
                files_to_sync.append(element)
        logging.debug("Total de archivos a sync: %s" %len(files_to_sync))
        return files_to_sync


    ## Dado un nombre de usuario devuelve un array con la siguiente informacion
    # referente a la quota de nfs:
    #  - quota usada
    #  - limite de quota
    def get_quota_info(self):
        self.mount()
        quota_command = ["quota", "-wp", "-u", self.__user_name]
        (return_code, output) = self.execute_command(quota_command)
        if return_code>0:
            logging.error("Error consultando la quota")
            return [0, 0]

        # cogemos solo la tercera linea de la salida y nos quedamos con lo que
        # no sean espacios, algo así:
        # ['c0:/home/nfs/ssoto', '15548', '102400', '102400', '0', '6', '0', '0', '0']
        cute_output = None
        for line in output.split('\n'):
            if line.startswith('c0'):
                cute_output = line.split()
                break

        self.umount()

        if cute_output is None:
            return [0, 0]

        used   =   int(cute_output[1])
        limit = int(cute_output[2])
        logging.debug("la informacion de %s(%s:%s) es: used==>%s limit==>%s"
                      %(self.__user_name,self.__user_id,self.__user_guid,
                        used,limit))
        return [used, limit]


    def save_config_dirs(self, dirs):
        logging.debug("save_config_dirs: %s " % dirs)
        self.mount()

        dst = os.path.join(self.config_tmp_dir, self.__user_name)
        org = self.mount_point

        if not os.path.exists(dst):
            os.makedirs(dst)
        else:
            shutil.rmtree( dst)
            os.makedirs(dst)
        os.chown(dst, self.__user_id, self.__user_guid)

        os.setegid(self.__user_guid)
        os.seteuid(self.__user_id)
        for dir in dirs:
            logging.debug("guardando configuracion de %s (%s/%s)"
                          %(dir, dirs.index(dir)+1, len(dirs)))
            abs_dir = os.path.join(org, dir)
            if os.path.exists(abs_dir):
                if( dir == '.evolution'):
                    # coger salida de gconftool-2 --dump /apps/evolution
                    # y meter en $HOME/.evolution/backup-restore-gconf.xml
                    gconf_file = os.path.join(dst,
                                              "backup-restore-gconf.xml")
                    try:
                        output = subprocess.check_output(
                                     ["gconftool-2", "--dump", "/apps/evolution"])
                    except CalledProcessError as exc:
                        logging.error("Error haciendo copia de evolution, gconftool-2 devuelve codigo de error: %s"
                                      %exc.returncode)
                        break
                    else:
                        with open( gconf_file, 'w') as file_d:
                            logging.debug("guardando configuracion gconftool en %s"
                                          %gconf_file)
                            file_d.write(output)
                logging.debug("Copiando archivo de configuración de dir: de %s a %s"
                              %( abs_dir.encode('utf-8'), dst.encode('utf-8')))
                shutil.move(abs_dir, dst)
        os.seteuid(0)
        os.setegid(0)
        self.umount()

    def create_if_should(self, folder):
        if not os.path.exists(folder):
                        os.makedirs(folder)
                        os.chown(folder,
                                 self.__user_id,
                                 self.__user_guid)



    def restore_config_dirs(self):
        origin = os.path.join(self.config_tmp_dir, self.__user_name)
        destiny = self.__home_dir

        if not os.path.exists(origin):
            logging.debug("no existe configuracion para el usuario %s que restaurar: %s"
                          %(self.__user_name, origin))
            return False

        #
        #    Empezamos con la manteca
        #
        dirs =  os.listdir(origin)
        configs = [".thunderbird", ".mozilla", "chromium"]
        for config in configs:
            if config == ".thunderbird" and config in dirs:
                # Configuracion para    thunderbird
                logging.debug("restaurando %s" %config)
                try:
                    result = self.move_and_chown(config, origin, destiny)
                except Exception as inst:
                    logging.error("error en la configurando de %s, motivo: %s"
                                  %(config, repr(inst)))
                    return False
                if not result:
                    logging.error("error restaurando %s" %config)
                else:
                    logging.debug("configurado satisfactoriamente    %s" %config)
            elif config == ".mozilla" and config in dirs:
            # Configuracion para    firefox
                logging.debug("restaurando %s" %config)
                try:
                    result = self.move_and_chown(config, origin, destiny)
                except Exception as inst:
                    logging.error("error en la configurando de %s, motivo: %s"
                                  %(config, repr(inst)))
                    return False
                if not result:
                    logging.error("error restaurando %s" %config)
                else:
                    logging.debug("configurado satisfactoriamente    %s" %config)
            elif config == 'chromium'and config in dirs:
                # Configuracion para    Chromium
                logging.debug("restaurando %s" %config)
                # no se copia todo, solo lo correspondiente a bookmarks
                backup_dir = os.path.join(origin, "chromium/Default")
                chromium_dir = os.path.join(destiny, ".config/chromium/Default")
                # en el caso de no existir el .config lo creamos,
                # pero uno a uno!
                self.create_if_should(os.path.join(destiny,
                                                   ".config"))
                self.create_if_should(os.path.join(destiny,
                                                   ".config/chromium"))
                self.create_if_should(os.path.join(destiny,
                                                   ".config/chromium/Default"))
                result = self.move_and_chown("Bookmarks", backup_dir, chromium_dir)
                if not result:
                    logging.error("error restaurando %s" %config)
                else:
                    logging.debug("configurado satisfactoriamente    %s" %config)
        # y borramos el original
        shutil.rmtree(origin)
        return True



    ## Mueve y cambia de propietario (recursivamente) un elemento. El chown se
    # hará para cambiar a usuario/grupo que se pasa de parámetro al constructor
    #
    # @param file: archivo o carpeta a tratar
    # @param origin: lugar de origen del elemento
    # @param destiny: directorio destino del elemento
    def move_and_chown(self, file, origin, destiny):
        if not os.path.isdir(origin):
            logging.error("no existe origen %s" %origin)
            return False
        elif  not os.path.isdir(destiny):
            logging.error("no existe destino para mover %s" %destiny)
            return False
        elif not os.path.exists(os.path.join(origin,file)):
            logging.error("no existe el elemento a mover %s" %os.path.join(origin, file))
            return False
        else:
            shutil.move(os.path.join(origin,file),
                        os.path.join(destiny, file))
            os.chown(os.path.join(destiny,file),
                     self.__user_id,
                     self.__user_guid)
            return True
